;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; COPYRIGHT (C):     1988 Horstmann Software. All Rights Reserved.
; PROJECT:	     Chiwriter
; FILE:		     screen.asm
; PURPOSE:	     ChiWriter Screen Driver
; VERSION:	     3.10
; LANGUAGE:	     8086 Assembler
; TARGET:	     IBM PC
; PROGRAMMER:	     Cay Horstmann (CSH), Keith Watson (KW)
; 
; UPDATE HISTORY:    1/17/89
;                    KW.  Wrote screen save and restore services
;                    for Chiwriter version 4.0
;		     10/20/89
;		     CSH. Incorporated Shirley Hwang's changes for quarter
;		     steps
;		     12/13/91
;                    CSH. Adapted to reading args from stack.
;		     5/26/91
;		     CSH. Added put service.
;		     NOTE: MUST ADD EGA IMPLEMENTATION!!!!!!
;		     2/9/92
;		     HM.  Added code recommended in MS Mouse reference for
;		     displaying mouse cursor in Hercules: If CRT page is 0,
;		     store a 6 in memory location 40H:49H, store a 5 if CRT
;		     page is 1
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; To make this into a driver file, assemble it--
; 	MASM -DscreenType -Dmode SCREEN.ASM;
; e.g. 
;	MASM -DCGA -DLO SCREEN.ASM;
; link it--
;	LINK SCREEN;
; ignore the "missing stack segment" and convert into a .COM file--
; 	EXE2BIN SCREEN.EXE <driver name>.DRV
;
; Supported graphics boards are
;
; CGA		-DCGA
; EGA		-DEGA
; VGA		-DVGA
; Super VGA	-DSVGA
; Hercules	-DHERC
; Tecmar	-DTEC
; AT&T 6300	-DATT
; Toshiba 	-DTOSH
; Wyse 700	-DWYSE
; Sigma 400	-DSIG
; Tandy 2000	-DTAN
;
; Comments on graphics boards:
; 
; (1) CGA
; No, you cannot change the colors on CGA. 
;
; (2) Wyse 700
; The drivers on the support disk use the 640 x 400 mode. 
; Assembling with -DWYSE -DXHI selects the WYSE 700 1280 x 800 display mode.
; Use the .PFT fonts (16 x 24 box) in that mode.
;   
; (3) Tecmar graphics board
; The Tecmar driver has been adapted by Prof. Leo Jonker from Queen's 
; University, Kingson, Ontario. He gives the following comments:
;      Chiwriter uses the first half of segment B000H as video buffer, and
; writes to the MONOCHROME screen in the 720 x 352 pixel resolution 2 -
; intensity graphics mode described in the TECMAR manual.  To operate in
; this mode the jumper settings on the TECMAR board should be as follows:
; 
;        JPR1:   Position B must be on.  The other positions may be on
;                or off depending on other software you may wish to run.
;        JPR4:   On.
;        JPR5:   Position B.
;        JPR6:   On.
;        JPR7:   On.
;        JPR8:   Off.;
;
; (4) Sigma 400
; The Sigma driver has been adapted by 	Timothy Williams, Physics Department
; College of William and Mary, Williamsburg, VA 23185. He writes:
; The driver writes to only one of the four memory planes on the Sigma board, 
; which turns out to be the green plane; this means that ChiWriter runs 
; in green.
;
;---------------------------------------------------------------------------
;
; The following services are provided:
;
; GRAPHICS BOARD CALLS
;
; (*gr_go)(code,args...)
;
; (*gr_go)(0,bg,fg)
; initialize the graphics board, clear screen and switch to text mode
;
; (*gr_go)(1)
; terminate and prepare the graphics board to return to DOS
;
; (*gr_go)(2)
; switch to text mode and clear screen 
;
; (*gr_go)(3)
; switch to graphics mode and clear screen
;
; (*gr_go)(4,h,v,dh,p)
; put a pattern row on the screen
;
; h = pixel row of pattern 
; v = leftmost pixel col of character
; dh = length of pattern in bit
; p = address of char
;
; (*gr_go)(5,p,r,c,rtop,rbot)
; or a character on the edit screen
;
; (*gr_go)(6,p,r,c,rtop,rbot)
; xor a character on the edit screen
;
; r = bottom edit row of char
; c = edit col of char
; rtop = top row of edit screen that is permitted to change
; rbot = bottom row of edit screen that is allowed to change
; p = address of char 
;
; (*gr_go)(7,r1,c1,r2,c2)
; clear an area on the edit screen
;
; (*gr_go)(8,r1,c1,r2,c2,dr,dc)
; move an area on the edit screen
;
; (*gr_go)(9,r1,c1,r2,c2,p)
; save an area on the screen into a buffer
;
; (*gr_go)(10,r1,c1,r2,c2,p)
; restore saved buffer to screen 
;
; r1 = top row 
; r2 = bot row
; c1 = left col
; c2 = right col
; p = long pointer to buffer in which to store immage
;
; (*gr_go)(11,p,r,c,rtop,rbot)
; put a character on the edit screen (like services 5,6)
;
;---------------------------------------------------------------------------
;
; The following services are provided:
;
;       Initialize graphics board. Switch to text mode & clear screen
;
; AH = 0
;
;       Terminate. Reset graphics board for return to DOS
;
; AH = 1
;
;       Switch to text mode and clear screen
;
; AH = 2
;
;       Switch to graphics mode and clear screen
;
; AH = 3
;
;       Put a bit pattern on the screen
;
; AH = 4
; BX = pixel row of pattern 
; CX = leftmost pixel col of character
; DX = length of pattern in bit
; DS:SI = address of char
;
;       Or a character on the edit screen
;
; AH = 5
; BX = bottom edit row of char
; DI = edit col of char
; DX = top row of edit screen that is permitted to change
; CX = bottom row of edit screen that is allowed to change
; DS:SI = address of char 
;
;       Xor a character on the edit screen
;
; AH = 6
; BX = bottom edit row of char
; DI = edit col of char
; DX = top row of edit screen that is permitted to change
; CX = bottom row of edit screen that is allowed to change
; DS:SI = address of char 
;
;       Delete an area on the edit screen
;
; AH = 7
; BH = r1
; BL = r2
; CH = c1
; CL = c2
;
;       Move an area on the edit screen
;
; AH = 8
; BH = r1
; BL = r2
; CH = c1
; CL = c2
; DH = dr
; DL = dc
;
;       Save an area on the screen into a buffer    (KW)
;
; AH = 9
; BH = r1   top row 
; BL = r2   bot row
; CH = c1   left col
; CL = c2   right col
; DS:DI = long pointer to buffer in which to store immage
;
;       Restore saved buffer to screen   (KW)
;
; AH = 10
; BH = r1   top row 
; BL = r2   bot row
; CH = c1   left col
; CL = c2   right col
; DS:SI = long pointer to buffer to restore immage from. 
;         
;
; For the screen save and restore services, the caller is totally
; responsible for consistency of screen coordinates and buffer pointers
; between calls.  If caller maintains a stack of saved objects, any number
; of saves can be performed and then undone in the correct order.
; 
;
;
; Note that each character is 8 bit wide and 16 scan rows high.
; Each ChiWriter row is 8 scan rows high.
;
;
; Screen memory map example 
; SCAN_BLKS = 4, BLK_DIFF = 2000H
;
; +--------------------------------+
; | xxxxxxxxxxx unused xxxxxxxxxxx |
; |				   |
; |   scan lines 3, 7, ... 399     |
; +--------------------------------+ <--- 6000H
; | xxxxxxxxxxx unused xxxxxxxxxxx |
; |				   |
; |   scan lines 2, 6, ... 398     |
; +--------------------------------+ <--- 4000H
; | xxxxxxxxxxx unused xxxxxxxxxxx |
; |				   |
; |   scan lines 1, 5, ... 397     |
; +--------------------------------+ <--- 2000H
; | xxxxxxxxxxx unused xxxxxxxxxxx |
; |				   |
; |   scan lines 0, 4, ... 396     |
; +--------------------------------+ <--- 0000H
;
;****************************************************************************
; Equates
;****************************************************************************
;
;
ifdef CGA
SC_VSIZE EQU  	200	; vertical screen size in pixels
VIDEO_SEG EQU 	0B800H 	; start of video segment
SCAN_DIFF EQU 	80	; the distance between successive scan lines
BLK_DIFF  EQU  	2000H	; the difference between successive scan line blocks
SCAN_BLKS EQU 	2 	; the number of scan line blocks
GRMODE	EQU	6	; BIOS graphics mode
endif

ifdef VGA
SC_VSIZE EQU  	480	; vertical screen size in pixels
VIDEO_SEG EQU 	0A000H 	; start of video segment
SCAN_DIFF EQU 	80	; the distance between successive scan lines
BLK_DIFF  EQU  	0	; the difference between successive scan line blocks
SCAN_BLKS EQU 	1 	; the number of scan line blocks
GRMODE	EQU	11H	; BIOS graphics mode
COLOR   EQU     1
endif

ifdef SVGA
SC_VSIZE EQU  	600	; vertical screen size in pixels
VIDEO_SEG EQU 	0A000H 	; start of video segment
SCAN_DIFF EQU 	100	; the distance between successive scan lines
BLK_DIFF  EQU  	0	; the difference between successive scan line blocks
SCAN_BLKS EQU 	1 	; the number of scan line blocks
GRMODE	EQU	6AH	; BIOS graphics mode
COLOR   EQU     1
endif

ifdef EGA
SC_VSIZE EQU  	480	; vertical screen size in pixels
VIDEO_SEG EQU 	0A000H 	; start of video segment
SCAN_DIFF EQU 	80	; the distance between successive scan lines
BLK_DIFF  EQU  	0	; the difference between successive scan line blocks
SCAN_BLKS EQU 	1 	; the number of scan line blocks
COLOR   EQU     1
endif

ifdef HERC
SC_VSIZE EQU  	348	; vertical screen size in pixels
VIDEO_SEG EQU 	0B000H 	; start of video segment
SCAN_DIFF EQU 	90	; the distance between successive scan lines
BLK_DIFF  EQU  	2000H	; the difference between successive scan line blocks
SCAN_BLKS EQU 	4 	; the number of scan line blocks
endif

ifdef TEC
SC_VSIZE EQU  	348	; vertical screen size in pixels
VIDEO_SEG EQU 	0B000H 	; start of video segment
SCAN_DIFF EQU 	90	; the distance between successive scan lines
BLK_DIFF  EQU  	2000H	; the difference between successive scan line blocks
SCAN_BLKS EQU 	4 	; the number of scan line blocks
endif

ifdef ATT
SC_VSIZE EQU  	400	; vertical screen size in pixels
VIDEO_SEG EQU 	0B800H 	; start of video segment
SCAN_DIFF EQU 	80	; the distance between successive scan lines
BLK_DIFF  EQU  	2000H	; the difference between successive scan line blocks
SCAN_BLKS EQU 	4 	; the number of scan line blocks
GRMODE	EQU	40H	; BIOS graphics mode
endif

ifdef TOSH
SC_VSIZE EQU  	400	; vertical screen size in pixels
VIDEO_SEG EQU 	0B800H 	; start of video segment
SCAN_DIFF EQU 	80	; the distance between successive scan lines
BLK_DIFF  EQU  	2000H	; the difference between successive scan line blocks
SCAN_BLKS EQU 	4 	; the number of scan line blocks
GRMODE	EQU	74H	; BIOS graphics mode
endif

ifdef SIG
SC_VSIZE EQU  	400	; vertical screen size in pixels
VIDEO_SEG EQU 	0B800H 	; start of video segment
SCAN_DIFF EQU 	80	; the distance between successive scan lines
BLK_DIFF  EQU  	2000H	; the difference between successive scan line blocks
SCAN_BLKS EQU 	4 	; the number of scan line blocks
GRMODE	EQU	42H	; BIOS graphics mode
endif

ifdef TAN
SC_VSIZE EQU  	400	; vertical screen size in pixels
VIDEO_SEG EQU 	0E000H 	; start of video segment
SCAN_DIFF EQU 	80	; the distance between successive scan lines
BLK_DIFF  EQU  	0	; the difference between successive scan line blocks
SCAN_BLKS EQU 	1 	; the number of scan line blocks
GRMODE	EQU	9	; BIOS graphics mode
endif

ifdef WYSE

ifdef XHI
SC_VSIZE EQU  	400	; vertical screen size in pixels
VIDEO_SEG EQU 	0A000H 	; start of video segment
SCAN_DIFF EQU 	160	; the distance between successive scan lines
BLK_DIFF  EQU  	0	; the difference between successive scan line blocks
SCAN_BLKS EQU 	1 	; the number of scan line blocks
else
SC_VSIZE EQU  	400	; vertical screen size in pixels
VIDEO_SEG EQU 	0A000H 	; start of video segment
SCAN_DIFF EQU 	80	; the distance between successive scan lines
BLK_DIFF  EQU  	0	; the difference between successive scan line blocks
SCAN_BLKS EQU 	1 	; the number of scan line blocks
endif
endif

;
ifdef HI
CH_VSIZE EQU	8	; vertical half-char box size in pixels
endif
ifdef LO
CH_VSIZE EQU	6	; vertical half-char box size in pixels
endif
ifdef XHI
CH_VSIZE EQU	6	; vertical half-char box size in pixels
endif

ifdef XHI
CH_HBYTES EQU  	2	; horizontal char box size in bytes
SC_HSIZE EQU  	1280	; horizontal screen size in pixels
else
CH_HBYTES EQU  	1	; horizontal char box size in bytes

ifdef HERC
SC_HSIZE EQU  	720	; horizontal screen size in pixels
else
ifdef SVGA
SC_HSIZE EQU  	800	; horizontal screen size in pixels
else
SC_HSIZE EQU  	640	; horizontal screen size in pixels
endif
endif

endif

MAX_HPOS EQU  	SC_HSIZE/(8*CH_HBYTES) - 1	; maximal horizontal position
MAX_VPOS EQU	SC_VSIZE/(CH_VSIZE/2) - 1	; maximal vertical position

;****************************************************************************
; The offset corresponding to pixel coordinates (r,c) is
; 	offset = (r/SCAN_BLKS) * SCAN_DIFF + (r%SCAN_BLKS) * BLK_DIFF
;
; NOTE: If SCAN_BLKS divides SCAN_DIFF, this is computed as follows:
; 	mod = r%SCAN_BLKS;
; 	offset = (r - mod) * (SCAN_DIFF/SCAN_BLKS) + mod * SCAN_BLKS;
;
;****************************************************************************
; MACROS
;****************************************************************************
if1
;
shiftdiv MACRO reg,pow2

if pow2 ge 16
	SHR reg,1
endif
if pow2 ge 8
	SHR reg,1
endif
if pow2 ge 4
	SHR reg,1
endif
if pow2 ge 2
	SHR reg,1
endif

	ENDM
;****************************************************************************
scanaddr MACRO
	LOCAL	ADD_LOOP, ADD_DONE
;
; gets BX = scan line number, adds offset to DI
;
ifdef XHI
	ADD	DI,DI			; fix up col
endif
if scan_blks gt 1
	PUSH	BX
	PUSH	CX
	MOV	CX,SCAN_BLKS - 1
	AND	CX,BX
	JZ	ADD_DONE
	SUB	BX,CX		; now bx has last bits==0
ADD_LOOP: ADD	DI,BLK_DIFF
if scan_blks gt 2
	LOOP	ADD_LOOP
endif
ADD_DONE:
	POP	CX
endif
;
	PUSH	DX
if (scan_diff mod scan_blks) eq 0
	MOV	AX,SCAN_DIFF/SCAN_BLKS
endif
if (scan_diff mod scan_blks) ne 0
	shiftdiv BX,SCAN_BLKS	; Here we assume SCAN_BLKS is a power of 2	
	MOV	AX,SCAN_DIFF
endif
	MUL	BX
	POP	DX
	ADD	DI,AX
;
if scan_blks gt 1
	POP	BX
endif
;
	ENDM
;****************************************************************************
row2scan MACRO	reg
;
; Receives reg = row. Sets BX = scan line
;
if CH_VSIZE EQ 8
ifdifi <reg>,<bl>
	MOV	BL,reg
endif
	XOR	BH,BH
	SHL	BX,1
	SHL	BX,1 		; scan line = 4 * reg (SH)
else
	MOV	AL,CH_VSIZE
	MUL	reg
	SHR	AX, 1		; (SH)
	MOV	BX,AX		; scan line = CH_VSIZE * reg / 2
endif
	ENDM
;****************************************************************************
incscan	MACRO	reg
	LOCAL	INC_BACK,INC_DONE
;
if scan_blks gt 1
	CMP	reg,BLK_DIFF*(SCAN_BLKS-1)
	JAE	INC_BACK
	ADD	reg,BLK_DIFF	; move on to next block
	JMP	INC_DONE
INC_BACK:	
endif
	ADD 	reg,SCAN_DIFF - BLK_DIFF*(SCAN_BLKS-1)
INC_DONE:
;
	ENDM
;****************************************************************************
decscan	MACRO	reg
	LOCAL	DEC_FRWD,DEC_DONE
if scan_blks gt 1
	CMP	reg,BLK_DIFF
	JB	DEC_FRWD
	SUB	reg,BLK_DIFF
	JMP	DEC_DONE
DEC_FRWD:
endif	
	SUB	reg,SCAN_DIFF - BLK_DIFF*(SCAN_BLKS-1)
DEC_DONE:
;
	ENDM
;****************************************************************************
transfer MACRO	instr,reg
	LOCAL	T_LOOP, T_NOINV, T_CONT
T_LOOP:	
ifdef EGA
	MOV	reg,ES:[DI]	; get into registers
endif
	MOV	reg,[SI]

ifndef COLOR
	cmp	cs:bg_color,0
	je	T_NOINV
ifdef XHI
	XOR	WORD PTR ES:[DI],0FFFFH
else
	XOR	BYTE PTR ES:[DI],0FFH
endif
	instr	ES:[DI],reg
ifdef XHI
	XOR	WORD PTR ES:[DI],0FFFFH
else
	XOR	BYTE PTR ES:[DI],0FFH
endif
	jmp	T_CONT
T_NOINV:instr	ES:[DI],reg
T_CONT: 
else
	instr	ES:[DI],reg
endif

ifdef XHI
	ADD	SI,4
else
	INC	SI
endif
	incscan	DI
	LOOP	T_LOOP	
	ENDM
;****************************************************************************
setchar MACRO	instr
	LOCAL	S_RET1, S_RET, S_OK0, S_OK1, S_OK2, S_OK3, S_LOOP
	LOCAL	S_Q1, S_Q2, S_Q3, S_Q4, S_Q5
;
        PUSH    BP
	PUSH	CX	; (SH)
	PUSH	DX
	PUSH	AX
	PUSH	BX
;
	CMP	DI,MAX_HPOS
	JG	S_RET1		; col > max. horiz. position ? return
	OR	DI,DI
	JL	S_RET1		; col < 0 ? return

	MOV	BP,2*CH_VSIZE	; BP will contain the number of scan lines

	cmp	dx, cx
	jg	S_RET1		; invalid window

	mov	ax, bx		;\
	cmp	ax, dx		; \
	jl	S_RET1		;  \
	sub	ax, 3		;   > check see if completely out of screen
	cmp	ax, cx		;  /
	jle	S_OK0		; /
S_RET1:	JMP	S_RET

S_OK0:	cmp	ax, dx		; check top
	jge	S_OK1

	push	dx		; clip top
	sub	dx, ax
	mov	ax, CH_VSIZE / 2	; # scans per quarter step
	mul	dx		
	add	si, ax		; increment source offset
	sub	bp, ax		; ax = # scan lines out of screen
	pop	ax

S_OK1:	cmp	bx, cx		; check bot
	jle	S_OK2

	xchg	ax, bx		; clip bot
	sub	ax, cx
	mov	cx, CH_VSIZE / 2	; # scans per quarter step
	mul	cx		; ax = # scan lines out of screen
	sub	bp, ax
	jmp	S_OK3		; row # already in bx

S_OK2:	mov bx, ax		; scan line expected in ax
;
S_OK3:	
	row2scan BL		; scan line in BX
	scanaddr	       	; offset added to DI
	MOV	CX,BP		; cx = #scan lines to display
;
ifdef XHI
	PUSH	SI
	PUSH	DI
	PUSH	CX
	transfer instr,AX
	POP	CX
	POP	DI
	POP	SI
	ADD	SI,2
	CALL	PAGE1
	transfer instr,AX
	CALL	PAGE0
else
	transfer instr,AL
endif
;
S_RET:	
	POP	BX		; (SH)
	POP	AX
	POP	DX
	POP	CX
	POP     BP
	ENDM
;****************************************************************************
makecounts MACRO errlabel
	LOCAL	M_OK1		; (SH) 
;
; Places BX = #scan rows, CX = #columns (from BH,BL,CH,CL)
;
	SUB	CL,CH
	INC	CL		; ccount=c2-c1+1
	JLE	errlabel	; ccount <= 0 --return
	XOR	CH,CH	      	; CX = # cols to move
;
	CMP	BL,BH		; rcount <= 0 --return
	JB	errlabel
	SUB	BL,BH
	INC	BL		; rcount=r2-r1+1
	MOV	AL,CH_VSIZE	;
	MUL	BL

	SHR	AX, 1		; #scans divided by 2 (SH)
if CH_VSIZE EQ 5
	TEST	BL, 01H		; Is BL even number? (SH)
	JZ	M_OK1
	TEST	BH, 01H		; Is BH even number?
	JZ	M_OK1
	INC	AX		; if BH is odd add 1 to AX
M_OK1:	MOV	BX, AX		; BX = #scans to move
else
	MOV	BX,AX		; BX = #scans to move
endif
;
	ENDM
;
endif				; of if1
;****************************************************************************
;
CODESEG SEGMENT PUBLIC 'CODE'
        ASSUME  CS:CODESEG
;
;
@AB	EQU	6		; large code model	
;
SCREENDR PROC	FAR
;
	PUSH	BP
	MOV	BP,SP
	push	si
	push	di
	PUSH	DS
	PUSH	ES

	MOV	AX,VIDEO_SEG
	MOV	ES,AX
;
	MOV	BX,@AB[BP]	; service number

        CMP     BX,0		; branch to appropriate service
	JL	SBADSRV
	CMP	BX,(SJMPEND-SJMPTBL)/2
	JG	SBADSRV
	ADD	BX,BX
	CALL	WORD PTR CS:SJMPTBL[BX]
SBADSRV:
	POP	ES
	POP	DS
	pop	di
	pop	si
	cld

	POP	BP
	RET

SJMPTBL LABEL	WORD
	DW	INIT
        DW      TERM
        DW      TEXTMODE
        DW      GRAPHIC
        DW      PUTROW
        DW      ORCHAR
        DW      XORCHAR
        DW      ERASE
        DW      MOVE
        DW      SAVE
        DW      RESTORE
	DW	PUTCHAR
SJMPEND LABEL	WORD

SCREENDR ENDP
;
;-------------------------------------------------------------------
;

ifdef HERC
;
; port addresses
INDEX   EQU     03B4H
CNTRL   EQU     03B8H
CONFIG  EQU     03BFH
;
; control codes
SCRN_ON EQU     8
GRPH    EQU     2
TEXT    EQU     20H
;
GTABLE  DB      35H,2DH,2EH,07H
        DB      5BH,02H,57H,57H
        DB      02H,03H,00H,00H
TTABLE  DB      61H,50H,52H,0FH
        DB      19H,06H,19H,19H
        DB      02H,0DH,0BH,0CH
; Hin 2/9/92 : Holds the value at address 40H:49H required in displaying
;	mouse cursor in Hercules
HERCADD DB 0

endif

ifdef TEC
;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
; The Tecmar manual refers to the "index register" as the POINTER
; REGISTER, so I renamed it PNTREG here. Similarly the control register
; is called the MODE SELECT REGISTER in the Tecmar manual as well as in
; the IBM technical reference manual, so I renamed it MSR.  Tecmar has
; an additional mode select register not found on the IBM graphics card,
; nor apparently the Hercules card.  It is called the EXTENDED MODE
; SELECT REGISTER ( XMSR ).  CSR refers to the COLOUR SELECT REGISTER
; which for some reason or other has to be set to a non zero value for
; this graphics mode, though not for some of the other Tecmar graphics
; modes.  Tecmar does not use the i/o address 03BFH.
;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
;
; port addresses
PNTREG  EQU     03B4H
MSR     EQU     03B8H
XMSR    EQU     03BAH
CSR     EQU     03B9H
;
; control codes
SCRN_ON EQU     8      ;used to set the video on bit in the MSR
GRPH    EQU     18     ;number for MSR but with video bit = 0
TEXT    EQU     1      ;number for MSR but with video bit = 0
XTEXT   EQU     0      ;number for XMSR / text mode
XGRPH   EQU     50     ;number for XMSR / graphics mode
CSR_ON  EQU     1      ;sets colour select to a non-zero value for
                       ;graphics
;
GTABLE  DB      54,45,45,08   ;These numbers go to the 6845 CRT con-
        DB      91,01,86,88   ;troller's registers R0,R1,...,R11 in that
        DB      02,03,32,00   ;order, to set the 720x352 2-intensity
                              ;graphics mode.
;
TTABLE  DB      97,80,81,15   ;These numbers are used the same way to
        DB      25,04,25,25   ;set the 80x25 alphanumeric mode.
        DB      02,13,11,12
endif

;
;-------------------------------------------------------------------
;
;
INIT	PROC	NEAR

ifdef HERC
        MOV     DX,CONFIG
        MOV     AL,1
        OUT     DX,AL

; Hin added the following code to save the old value at address 40H:49H
	PUSH	BX
	PUSH    CX
	PUSH   	DS

	MOV	BX, 40H
	MOV 	DS, BX 
	MOV	BX, 49H

	MOV	CH, BYTE PTR[BX]	; move value at 40H:49H to CH
	POP	DS			; restores DS
	MOV 	HERCADD, CH		; saves old value at 40H:49H with HERCADD
	POP	CX
	POP	BX
;Hin

endif

	mov	bl,@AB[bp+2]
	mov	cl,@AB[bp+4]
	MOV	CS:BG_COLOR,BL
	MOV	CS:FG_COLOR,CL

ifdef GRMODE
	mov	bx,@AB[bp+6]
	MOV	CS:GR_MODE,BX
endif
	CALL	TEXTMODE
	RET
INIT	ENDP
;
BG_COLOR DB 	0
FG_COLOR DB	0
ifdef GRMODE
GR_MODE DW	0
endif
;
;-------------------------------------------------------------------
;
TERM	PROC	NEAR

ifdef COLOR
        MOV     AH,0			; don't call TEXTMODE--it sets palette
        MOV     AL,3
        INT     10H
else	
	CALL	TEXTMODE
endif
	RET
TERM	ENDP
;
;-------------------------------------------------------------------

ifdef COLOR

PALETTE PROC	NEAR
; receives: BL = color for foreground
; 7 for text, 15 for graphic
	PUSH	AX
	PUSH	BX
	MOV	AX,1000H
	MOV	BH,cs:fg_color
	INT	10H
	MOV	AX,1000H
	MOV	BL,0
	MOV	BH,cs:bg_color
	INT	10H
	POP	BX
	POP	AX
	RET
PALETTE ENDP

endif

;
;-------------------------------------------------------------------
;
GRAPHIC PROC    NEAR
;

;Hin 2/9/92: stores a 6 in memory location 40H:49H for Hercules
ifdef HERC
	PUSH	BX
	PUSH   	DS

	MOV	BX, 40H
	MOV 	DS, BX 
	MOV	BX, 49H

	MOV	BYTE PTR[BX], 6
	POP	DS
	POP	BX
endif
;Hin

ifdef GRMODE
ifdef SVGA			; would make sense for AT&T/Toshiba as well!
        MOV     AX,GR_MODE
	cmp	ax,0  		; no value set in screen parameter file
	jz	gr_default
	mov	bl,2		; hack: EVEREX needs al = 70h, bl = 2 for 800 x 600
	jmp	gr_skip
gr_default:
endif
	mov	al,GRMODE
gr_skip:
	mov	ah,0
        INT     10H		; use BIOS call to switch graphics mode
endif

ifdef EGA
	MOV	AH,12H
	MOV	BL,10H	; get EGA information
	INT	10H
	CMP	BL,0	; installed memory
	JNZ	G_EGA128
	MOV	BYTE PTR CS:EGA64,1
G_EGA128:CMP	BH,0
	JZ	GR_COLOR
        MOV     AL,0FH
	JMP	GR_SET
GR_COLOR:MOV    AL,10H
GR_SET: MOV     AH,0
        INT     10H
endif

ifdef HERC
        PUSH    ES
        PUSH    DS
        MOV     AX,CS
        MOV     DS,AX
        MOV     AL,GRPH
        LEA     SI,GTABLE
        MOV     BX,0
        MOV     CX,4000H
        CALL    SETMD
        POP     DS
        POP     ES
endif

ifdef TEC
        PUSH    ES
        PUSH    DS
; Load the Color Select Register with a non-zero value
        MOV     DX,CSR
        MOV     AL,CSR_ON
        OUT     DX,AL
; Ready parameters for a call to SETMD
        MOV     AX,CS
        MOV     DS,AX
        MOV     AL,GRPH
        MOV     AH,XGRPH
        LEA     SI,GTABLE
        MOV     BX,0         ; Zero to blank screen
        MOV     CX,4000H     ; 720d x 352d / 8 < 8000H bytes
                             ; = 4000H words.
        CALL    SETMD
        POP     DS
        POP     ES
endif

ifdef WYSE
ifdef XHI
	MOV	DX,3DFH
	MOV	AL,0C8H
	OUT	DX,AL
;
else
	MOV	DX,3DFH
	MOV	AL,088H
	OUT	DX,AL
endif
;
	MOV	DX,3DDH
	XOR	AL,AL
	OUT	DX,AL
	INC	DX
	OUT	DX,AL
	MOV	AX,VIDEO_SEG
	MOV	ES,AX
	XOR	AX,AX
endif

ifdef COLOR
	MOV	BL,15
	CALL 	PALETTE
else
				; if not color and bg color selected, 
				; fill screen
				; on Wyse, always fill screen
	cmp	cs:bg_color,0
ifdef WYSE
	je	GR_NOINV
else
	je	GR_NOFILL
endif
	MOV	AX,0FFFFH
GR_NOINV:
	XOR	DI,DI
	MOV	CX,SC_VSIZE*(SC_HSIZE/16)
	REP	STOSW		; clear graphic screen
ifdef WYSE
ifdef XHI
	CALL	PAGE1
	XOR	DI,DI
	MOV	CX,SC_VSIZE*(SC_HSIZE/16)
	REP	STOSW		; clear graphic screen
	CALL	PAGE0
endif
endif
GR_NOFILL:
endif
        RET
;
ifdef EGA
EGA64	DB	0
endif
;
GRAPHIC ENDP
;
;-------------------------------------------------------------------
;
TEXTMODE    PROC    NEAR

ifdef HERC

;Hin 2/9/92: always restore the original value at 40H:49H in text mode.
	PUSH   	DS
	PUSH	BX
	PUSH    CX
	MOV	CH, HERCADD
	MOV	BX, 40H
	MOV 	DS,BX 
	MOV	BX, 49H
	MOV	BYTE PTR[BX], CH
	POP	CX
	POP	BX
	POP	DS
;Hin

        PUSH    ES
        PUSH    DS
        MOV     AX,CS
        MOV     DS,AX
        MOV     AL,TEXT
        LEA     SI,TTABLE
        MOV     BX,720H
        MOV     CX,2000
        CALL    SETMD
        mov     ah,2         ;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        xor     dx,dx        ; I don't know why, but the text mode doesn't
        xor     bh,bh        ; seem to work without this interrupt.
        int     10h          ;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        POP     DS
        POP     ES
else

ifdef TEC
        PUSH    ES
        PUSH    DS
; Set the color select register to zero
        MOV     DX,CSR
        XOR     AL,AL
        OUT     DX,AL
; Ready parameters for the call to SETMD
        MOV     AX,CS
        MOV     DS,AX
        MOV     AH,XTEXT
        MOV     AL,TEXT
        LEA     SI,TTABLE
        MOV     BX,720H      ; attribute byte + blank,
        MOV     CX,2000      ; count for clearing screen
        CALL    SETMD
        mov     ah,2         ;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        xor     dx,dx        ; I don't know why, but the text mode doesn't
        xor     bh,bh        ; seem to work without this interrupt.
        int     10h          ;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
        POP     DS
        POP     ES
else

        MOV     AH,0
        MOV     AL,3
        INT     10H

endif
endif

ifdef COLOR
	MOV	BL,7
	CALL 	PALETTE
endif

        RET
TEXTMODE    ENDP
;

ifdef HERC
;-------------------------------------------------------------------
;
SETMD   PROC    NEAR
        PUSH    DS
        PUSH    ES
        PUSH    AX
        PUSH    BX
        PUSH    CX
        MOV     DX,CNTRL
        OUT     DX,AL
        MOV     AX,DS
        MOV     ES,AX
        MOV     DX,INDEX
        MOV     CX,12
        XOR     AH,AH
PARMS:  MOV     AL,AH
        OUT     DX,AL
        INC     DX
        LODSB
        OUT     DX,AL
        INC     AH
        DEC     DX
        LOOP    PARMS
        POP     CX
        MOV     AX,0B000H
        CLD
        MOV     ES,AX
        XOR     DI,DI
        POP     AX
        REP     STOSW
        MOV     DX,CNTRL
        POP     AX
        ADD     AL,SCRN_ON
        OUT     DX,AL
        POP     ES
        POP     DS
        RET
SETMD   ENDP
endif

ifdef TEC
;-------------------------------------------------------------------
;
SETMD   PROC    NEAR
        PUSH    DS
        PUSH    ES
        PUSH    AX
        PUSH    BX
        PUSH    CX
; Send number to MSR
        MOV     DX,MSR
        OUT     DX,AL
; Send number to XMSR
        MOV     DX,XMSR
        MOV     AL,AH
        OUT     DX,AL
; Ready parameters for the loop that loads the registers of the 6845
        MOV     AX,DS
        MOV     ES,AX
        MOV     DX,PNTREG
        MOV     CX,12
        XOR     AH,AH
        CLD
; The loop loading the registers of the 6845
PARMS:  MOV     AL,AH
        OUT     DX,AL
        INC     DX
        LODSB
        OUT     DX,AL
        INC     AH
        DEC     DX
        LOOP    PARMS
; Clear the screen
        POP     CX
        MOV     AX,0B000H
        MOV     ES,AX
        XOR     DI,DI
        POP     AX        ; AX now contains the value BX had at entry
        REP     STOSW
        MOV     DX,MSR
        POP     AX
; Set bit 3 in the MSR on to enable video
        OR      AL,SCRN_ON
        OUT     DX,AL
        POP     ES
        POP     DS
        RET
SETMD   ENDP
endif

;
;-------------------------------------------------------------------
;
PUTROW  PROC    NEAR
	MOV	BX,@AB[BP+2]	; h
	MOV	CX,@AB[BP+4]	; v
	MOV	DX,@AB[BP+6]    ; dh
        MOV     SI,@AB[BP+8]
	MOV	DI,@AB[BP+10]
	MOV	DS,DI		; font pointer

        CALL    PIXADDR 	; compute addr of (BX,CX) in ES:DI
        AND     CL,7
        MOV     CH,8
PRLOOP: CMP     DX,0
        JLE     PRRET
        CMP     DX,8
        JGE     PRSKIP
        MOV     CH,DL
PRSKIP: 
	PUSH    CX
        MOV     CL,CH
        MOV     BH,0FFH
        SHR     BH,CL		; BH = 0...01...1 with CH zeroes
	POP	CX
;
        MOV     AH,[SI]
ifndef COLOR
	cmp	cs:bg_color,0
	je	PR_NOINV
	XOR	AH,0FFH
PR_NOINV:
endif
        XOR     AL,AL
        MOV     BL,0FFH
        ROR     AX,CL
        ROR     BX,CL
ifdef EGA
	NOT	BX
;
; higher byte first!!!
;
	PUSH	AX
	MOV	AL,BH
	CALL	WRMASK
	MOV	AH,ES:[DI]
	POP	AX
	MOV	ES:[DI],AH
	PUSH	AX
	MOV	AL,BL
	CALL	WRMASK
	MOV	AL,ES:[DI+1]
	POP	AX
        MOV     ES:[DI+1],AL
else
        AND     ES:[DI],BH      ; higher byte first!!!
        AND     ES:[DI+1],BL
        NOT     BX
        AND     AX,BX
        OR      ES:[DI],AH
        OR      ES:[DI+1],AL
endif
        SUB     DX,8
        INC     DI
        INC     SI
        JMP     PRLOOP
PRRET:  
ifdef EGA
	MOV	AL,0FFH
	CALL	WRMASK
endif
	RET
PUTROW  ENDP
;
;-------------------------------------------------------------------
;
; BH = 0...01...1 with CH zeroes
;
;
;
;-------------------------------------------------------------------
;
PIXADDR PROC    NEAR
;
; PURPOSE: Compute address from pixel coordinates
; RECEIVES: (BX,CX) = (pixel row, pixel col)
; RETURNS: ES:DI = screen address
;
	PUSH	AX
;
;
        MOV     DI,CX
        SHR     DI,1		; divide col number by 8
        SHR     DI,1
        SHR     DI,1
	scanaddr
;
	POP	AX
        RET
PIXADDR ENDP
;
;-------------------------------------------------------------------
;
ORCHAR 	PROC	NEAR

	MOV	BX,@AB[BP+6]	; row
	MOV	DI,@AB[BP+8]	; col
	MOV	SI,@AB[BP+2]
	MOV	CX,@AB[BP+4]
	MOV	DS,CX		; font pointer
        OR      CX,SI
	JNZ	OR_SKIP
        JMP     OR_RET  	; if pointer==NULL, skip
OR_SKIP:
	MOV	DX,@AB[BP+10]	; sc_top
	MOV	CX,@AB[BP+12]	; sc_bot

ifdef EGA
;
; set the writemode to OR
;
	MOV	AL,00010000B
	CALL	WRFUNC
;
ORSTART:
	setchar MOV
	XOR	AL,AL
	CALL	WRFUNC		; reset write mode
else
				; not EGA
	setchar	OR
endif
OR_RET:	
	RET

ORCHAR	ENDP
;
;-------------------------------------------------------------------
;
XORCHAR	PROC	NEAR

	MOV	BX,@AB[BP+6]	; row
	MOV	DI,@AB[BP+8]	; col
	MOV	SI,@AB[BP+2]
	MOV	CX,@AB[BP+4]
	MOV	DS,CX		; font pointer
        OR      CX,SI
	JNZ	XOR_SKIP
        JMP     OR_RET  	; if pointer==NULL, skip
XOR_SKIP:
	MOV	DX,@AB[BP+10]	; sc_top
	MOV	CX,@AB[BP+12]	; sc_bot

ifdef EGA
;
; set the writemode to XOR
;
	MOV	AL,00011000B
	CALL	WRFUNC
;
	JMP	ORSTART
else
				; not EGA
	setchar	XOR
endif
	RET

XORCHAR	ENDP
;
;-------------------------------------------------------------------
;
;
;-------------------------------------------------------------------
;
PUTCHAR	PROC	NEAR

	MOV	BX,@AB[BP+6]	; row
	MOV	DI,@AB[BP+8]	; col
	MOV	SI,@AB[BP+2]
	MOV	CX,@AB[BP+4]
	MOV	DS,CX		; font pointer
        OR      CX,SI
	JNZ	PUT_SKIP
        JMP     OR_RET  	; if pointer==NULL, skip
PUT_SKIP:
	MOV	DX,@AB[BP+10]	; sc_top
	MOV	CX,@AB[BP+12]	; sc_bot

ifdef EGA
;
; set the writemode to PUT 
;
	XOR	AL,AL
	CALL	WRFUNC
;
	JMP	ORSTART
else
				; not EGA
	setchar	MOV
endif
	RET

PUTCHAR	ENDP
;
;-------------------------------------------------------------------
;
MOVE	PROC	NEAR
;
ifdef EGA
	CMP	BYTE PTR CS:EGA64,0
	JZ	M_WRMOD1
	CMP	WORD PTR @AB[BP+12],0 	; dc = 0?
	JZ	M_WRMOD1
	JMP	M_NOWR1
M_WRMOD1:MOV	AL,1
	CALL	WRMODE	
;
M_NOWR1:
endif

	CALL CLIP
; Now 0 <= r1, r2 <= max_hpos <= 127, 0 <= c1, c2 <= max_vpos <= 127
; Also assume -max_vpos <= dr <= max_vpos, -max_hpos <= dc <= max_hpos
;
; argument layout:
;	@AB[BP+2] = r1
;	@AB[BP+4] = c1
;	@AB[BP+6] = r2
;	@AB[BP+8] = c2
;	@AB[BP+10] = dr
;	@AB[BP+12] = dc
;
	XOR	AX,AX
	SUB	AX,@AB[BP+10]	
	CMP	@AB[BP+2],AX 	; r1 + dr < 0
	JGE	M_OK1		; no
	MOV	@AB[BP+10],AX	; yes-- r1 = -dr
M_OK1:	MOV	AX,MAX_VPOS
	SUB	AX,WORD PTR @AB[BP+10]
	CMP	WORD PTR @AB[BP+6],AX		; r2 + dr > maximum vertical?
	JLE	M_OK2		; no
	MOV	WORD PTR @AB[BP+6],AX		; yes-- r2 = max_vpos - dr
M_OK2:	XOR	AX,AX	
	SUB	AX,WORD PTR @AB[BP+12]
	CMP	WORD PTR @AB[BP+4],AX		; c1 + dc < 0?
	JGE	M_OK3		; no
	MOV	WORD PTR @AB[BP+4],AX		; yes-- c1 = -dc
M_OK3:	MOV	AX,MAX_HPOS
	SUB	AX,WORD PTR @AB[BP+12]
	CMP	WORD PTR @AB[BP+8],AX		; c2 + dc > maximum horizontal?
	JLE	M_OK4		; no
	MOV	WORD PTR @AB[BP+8],AX		; yes-- c2 = max_hpos - dc
;
M_OK4:	
	MOV	BH,@AB[BP+2]	; r1
	MOV	CH,@AB[BP+4]	; c1
	MOV	BL,@AB[BP+6]	; r2
	MOV	CL,@AB[BP+8]	; c2
	MOV	DL,@AB[BP+12]	; dc

	CMP	WORD PTR @AB[BP+12],0
	JL	M_DCN		; dc negative
	STD			; dc positive--move downward
	MOV	AL,CL		; start from last column
	JMP	M_DCOK
M_DCN:	CLD			; move upward
	MOV	AL,CH		; start from 1st column
M_DCOK:	CMP	WORD PTR @AB[BP+10],0
	JL	M_DRN		; dr negative
	JZ	M_DRZ
	MOV	DH,1		; need to check later whether > 0
	JMP	M_DR1
M_DRZ:	MOV	DH,0		; EGA needs to know whether == 0
M_DR1:
	MOV	AH,BL		; start from last row + 1 
	INC	AH		; (we'll decrement the scan first)
	JMP	M_DROK
M_DRN:	MOV	DH,-1		; need to check later whether > 0
	MOV	AH,BH		; start from first row
;
M_DROK:	CALL	ADDR		; compute address of source 
	PUSH	ES
	POP	DS		; move from ES:DI to DS:SI
	MOV	SI,DI
	ADD	AH,BYTE PTR @AB[BP+10]
	ADD	AL,DL
	CALL	ADDR		; compute address of target
	makecounts M_RET
;
M_LOOP: 
	CMP	DH,0		; dr > 0?
	JL	M_UP		; if so, first move, then increment
	decscan	SI
	decscan	DI
M_UP:
	PUSH	SI
	PUSH	DI
	PUSH	CX

ifdef EGA
	CMP	DH,0
	JZ	M_NO_64
	CMP	DL,0
	JZ	M_NO_64
	CALL	M_64MOVE
	JMP	M_CONT
M_NO_64:
endif

ifdef XHI
	REP	MOVSW
else
	REP	MOVSB
endif
M_CONT:
	POP	CX
	POP	DI
	POP	SI

ifdef XHI
	CALL	PAGE1
	PUSH	SI
	PUSH	DI
	PUSH	CX
	REP	MOVSW
	POP	CX
	POP	DI
	POP	SI
	CALL	PAGE0
endif
;
	CMP	WORD PTR @AB[BP+10],0		; dr <= 0?
	JGE	M_DOWN		; if so, already decremented
	incscan	SI
	incscan	DI
;
M_DOWN:	DEC	BX
	JNZ	M_LOOP
;
M_RET:	CLD
ifdef EGA
	XOR	AL,AL
	CALL	WRMODE
endif
	RET
MOVE	ENDP
;
;-------------------------------------------------------------------
;
ifdef EGA
M_64MOVE PROC	NEAR
;
; Auxiliary to do the even/odd garbage in 64 K EGA's
;
; read a byte from plane 0 if even address, plane 1 if odd address
;
	PUSH	DX
M_64CONT:MOV	AL,4		; set read mode 0
	MOV	DX,3CEH
	OUT	DX,AL
	MOV	DX,3CFH
	MOV	AX,SI
	AND	AL,1		; set bit plane 
	OUT	DX,AL
	MOV	AL,DS:[SI]	; now byte in AL
	MOV	AH,ES:[DI]	; prime latches
	MOV	ES:[DI],AL	; write byte
;
	PUSHF
	POP	AX
	AND	AH,4		; direction flag
	JNZ	M_DECR
	INC	SI
	INC	DI
	JMP	M_INCR
M_DECR:	DEC	SI
	DEC	DI
M_INCR:	DEC	CX
	JNZ	M_64CONT
	POP	DX
	RET
M_64MOVE ENDP
endif
;
;-------------------------------------------------------------------
;
ERASE 	PROC	NEAR
;
	CALL	CLIP

	MOV	BH,@AB[BP+2]	; r1
	MOV	CH,@AB[BP+4]	; c1
	MOV	BL,@AB[BP+6]	; r2
	MOV	CL,@AB[BP+8]	; c2
;
;
	MOV	AH,BH
	MOV	AL,CH
	CALL	ADDR		; compute address in DI
	makecounts E_RET
;
	CLD

	XOR	AX,AX		; need AX for XHI only!
ifndef COLOR
	cmp	cs:bg_color,0
	je	E_NOINV
	MOV	AX,0FFFFH
E_NOINV:
endif
;
E_LOOP:	PUSH	DI
	PUSH	CX
ifdef EGA
	MOV	AH,ES:[DI]	; get attention of registers
endif

ifdef XHI
	REP	STOSW
else
	REP	STOSB
endif
	POP	CX
	POP	DI

ifdef XHI
	CALL	PAGE1
	PUSH	DI
	PUSH	CX
	REP	STOSW
	POP	CX
	POP	DI
	CALL	PAGE0
endif

	incscan	DI
	DEC	BX
	JNZ	E_LOOP
;
E_RET:	RET
ERASE	ENDP
;
;-------------------------------------------------------------------
;
SAVE PROC	NEAR
;
;	       save a screen block into buffer
;              
;
	CALL 	CLIP
	MOV	BH,@AB[BP+2]	; r1
	MOV	CH,@AB[BP+4]	; c1
	MOV	BL,@AB[BP+6]	; r2
	MOV	CL,@AB[BP+8]	; c2
        MOV     SI,@AB[BP+10]
	MOV	DI,@AB[BP+12]
	MOV	DS,DI		; buffer location

	PUSH	AX
                                ;make sure parms non zero before clipping
	MOV	AH,BH
	MOV	AL,CH
	CALL	ADDR		;compute address in DI
	makecounts S_RET
;	
        POP     AX
	CLD

        PUSH    DS
        PUSH    SI
        MOV     AX,ES	        ;move ES to DS
        MOV     DS,AX
       	MOV     AX,DI
        MOV     SI,AX
        POP     DI		;pop DS into ES 
        POP     ES

;
S_LOOP:PUSH	SI
	PUSH	CX
ifdef EGA
	MOV	AH,DS:[SI]	; get attention of registers
endif

ifdef XHI
	REP	MOVSW
else
	REP	MOVSB
endif
	POP	CX
	POP	SI

ifdef XHI
	CALL	PAGE1
	PUSH	SI
	PUSH	CX
	REP	MOVSW
	POP	CX
	POP	SI
	CALL	PAGE0
endif
	incscan	SI
        DEC	BX
	JNZ	S_LOOP
;
S_RET:	RET
SAVE    ENDP
;
;-------------------------------------------------------------------
;
RESTORE PROC	NEAR
;
;	       save a screen block into buffer
;              
;
	CALL	CLIP

	MOV	BH,@AB[BP+2]	; r1
	MOV	CH,@AB[BP+4]	; c1
	MOV	BL,@AB[BP+6]	; r2
	MOV	CL,@AB[BP+8]	; c2
        MOV     SI,@AB[BP+10]
	MOV	DI,@AB[BP+12]
	MOV	DS,DI		; buffer location

	PUSH	AX

	MOV	AH,BH
	MOV	AL,CH
	CALL	ADDR		;compute address in DI
	makecounts R_RET
;	
        POP     AX
	CLD

R_LOOP: PUSH	DI
	PUSH	CX
ifdef EGA
	MOV	AH,ES:[DI]	; get attention of registers
endif

ifdef XHI
	REP	MOVSW
else
	REP	MOVSB
endif
	POP	CX
	POP	DI

ifdef XHI
	CALL	PAGE1
	PUSH	DI
	PUSH	CX
	REP	MOVSW
	POP	CX
	POP	DI
	CALL	PAGE0
endif
	incscan	DI
        DEC	BX
	JNZ	R_LOOP
;
R_RET:	RET
RESTORE ENDP
;
;
;-------------------------------------------------------------------
;
;
CLIP	PROC	NEAR
;
; argument layout:
;	@AB[BP+2] = r1
;	@AB[BP+4] = c1
;	@AB[BP+6] = r2
;	@AB[BP+8] = c2
;
	CMP	WORD PTR @AB[BP+2],0		; r1 < 0
	JGE	C_OK1				; no
	MOV	WORD PTR @AB[BP+2],0		; yes-- r1=0
C_OK1:	CMP	WORD PTR @AB[BP+6],MAX_VPOS	; r2 > max. screen row?
	JLE	C_OK2				; no
	MOV	WORD PTR @AB[BP+6],MAX_VPOS	; yes-- r2=max. screen row
C_OK2:	CMP	WORD PTR @AB[BP+4],0		; c1 < 0?
	JGE	C_OK3				; no
	MOV	WORD PTR @AB[BP+4],0		; yes-- c1=0
C_OK3:	CMP	WORD PTR @AB[BP+8],MAX_HPOS	; c2 > max. screen col?
	JLE	C_OK4				; no
	MOV	WORD PTR @AB[BP+8],MAX_HPOS	; yes-- c2=max. screen col.
C_OK4:	RET
;
CLIP	ENDP
;
;-------------------------------------------------------------------
;
; ADDR computes the screen offset of (Chiwriter row-1, col-1) in (AH,AL)
; in DI
;
;
ADDR	PROC	NEAR
	PUSH	AX
	PUSH	BX
	MOV	DI,AX
	AND	DI,0FFH	
;
	row2scan AH
	scanaddr
	POP	BX
	POP	AX
	RET
ADDR	ENDP

;***************************************************************************
; EGA functions
;***************************************************************************
ifdef EGA
;
; Set the write function ( put, or, xor, and )
;
WRFUNC	PROC	NEAR
;
	PUSH	DX
	PUSH	AX
	MOV	AL,3
WROUT:	MOV	DX,3CEH
	OUT	DX,AL
	MOV	DX,3CFH
	POP	AX
	OUT	DX,AL
	POP	DX
	RET
WRFUNC	ENDP
;
; Set the write mode
;
WRMODE	PROC	NEAR
;
	PUSH	DX
	PUSH	AX
	MOV	AL,5
	JMP	WROUT
WRMODE	ENDP
;
;
; Set the write mode
;
WRMASK	PROC	NEAR
;
	PUSH	DX
	PUSH	AX
	MOV	AL,8
	JMP	WROUT
WRMASK	ENDP
endif

ifdef WYSE
ifdef XHI
PAGE0	PROC	NEAR
	PUSH	AX
	PUSH	DX
	MOV	DX,3DFH
	MOV	AL,0C8H
	OUT	DX,AL
	POP	DX
	POP	AX
	RET
PAGE0	ENDP
;
PAGE1	PROC	NEAR
	PUSH	AX
	PUSH	DX
	MOV	DX,3DFH
	MOV	AL,0CBH
	OUT	DX,AL
	POP	DX
	POP	AX
	RET
PAGE1	ENDP
endif
endif


;
CODESEG ENDS
	END
